@@ -107,9 +107,15 @@ update.trellis <-
              strip.left,
              sub,
              xlab,
-             xlim,
              ylab,
+             xlab.top,
+             ylab.right,
+             xlim,
              ylim,
+             xscale.components,
+             yscale.components,
+             axis,
+
              par.settings,
              plot.args,
              lattice.options,
@@ -158,7 +164,12 @@ update.trellis <-
     if (!missing(sub)) object$sub <- sub
     if (!missing(xlab)) object$xlab <- xlab
     if (!missing(ylab)) object$ylab <- ylab
+    if (!missing(xlab.top)) object$xlab.top <- xlab.top
+    if (!missing(ylab.right)) object$ylab.right <- ylab.right
     if (!missing(page)) object$page <- page
+    if (!missing(xscale.components)) object$xscale.components <- xscale.components
+    if (!missing(yscale.components)) object$yscale.components <- yscale.components
+    if (!missing(axis)) object$axis <- axis
     if (!missing(par.strip.text))
     {
         ## this only overwrites earlier things, leaves alone those
@@ -251,7 +262,36 @@ update.trellis <-
     if (!missing(legend))
     {
         if (is.null(legend)) object$legend <- NULL
-        else object$legend <- updateList(object$legend, legend)
+        else
+        {
+            ## Used to be:
+            ## object$legend <- updateList(object$legend, legend)
+            ## but that does not handle multiple components with same
+            ## name, which can happen when there are multiple 'inside'
+            ## components (lattice bug #2587).
+
+            ## What should happen in that case?  If there is one or
+            ## more 'inside' in old, but none in new, retain all in
+            ## the old.  If there is one in the replacement, and one
+            ## in the original, then the new should replace the old.
+            ## If there are multiple, remove all in the old and retain
+            ## all in the new. [One might argue that there should be
+            ## the option of retaining the old, but in that case the
+            ## user could add that manually to the new 'legend'. ]
+
+            winside <- which(names(legend) == "inside")
+            if (length(winside) ==  0L) # none in new
+                object$legend <- updateList(object$legend, legend)
+            else 
+            {
+                object$legend <- object$legend[ names(object$legend) != "inside" ]
+                new.insides <- legend[winside]
+                legend <- legend[-winside]
+                object$legend <- updateList(object$legend, legend)
+                object$legend <- c(object$legend, new.insides)
+            }
+        }
+        
     }
     if (!missing(key))
     {
@@ -320,12 +360,16 @@ update.trellis <-
         if (is.logical(yscales$alternating)) yscales$alternating <- if (yscales$alternating) c(1,2) else 1
         ## cannot possibly make sense for z
 
+        rep2IfNotNULL <- function(x) 
+        {
+            if (is.null(x)) NULL else rep(x, length.out = 2)
+        }
         for (nm in c("tck", "cex", "rot"))
         {
-            scales[[nm]] <- rep(scales[[nm]], length.out = 2)
-            xscales[[nm]] <- rep(xscales[[nm]], length.out = 2)
-            yscales[[nm]] <- rep(yscales[[nm]], length.out = 2)
-            zscales[[nm]] <- rep(zscales[[nm]], length.out = 2)
+            scales[[nm]] <- rep2IfNotNULL(scales[[nm]])
+            xscales[[nm]] <- rep2IfNotNULL(xscales[[nm]])
+            yscales[[nm]] <- rep2IfNotNULL(yscales[[nm]])
+            zscales[[nm]] <- rep2IfNotNULL(zscales[[nm]])
         }
 
         if (!is.null(scales$limits))
@@ -360,9 +404,9 @@ update.trellis <-
         object$y.scales[names(yscales)] <- yscales
         object$z.scales[names(zscales)] <- zscales
 
-        if (object$x.scales$relation == "same" && (is.list(object$x.scales$at) || is.list(object$x.scales$lab)))
+        if (object$x.scales$relation == "same" && (is.list(object$x.scales$at) || is.list(object$x.scales$labels)))
             stop("the at and labels components of scales may not be lists when relation = same")
-        if (object$y.scales$relation == "same" && (is.list(object$y.scales$at) || is.list(object$y.scales$lab)))
+        if (object$y.scales$relation == "same" && (is.list(object$y.scales$at) || is.list(object$y.scales$labels)))
             stop("the at and labels components of scales may not be lists when relation = same")
     }
 
